/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Media Wrap code.
 *
 * The Initial Developer of the Original Code is
 * Sunwan<SunwanCN@gmail.com>, King, Helscn, Dudu8995<Dudu8995@gmail.com>.
 * Portions created by the Initial Developer are Copyright (C) 2006-2008
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */

var mediawrapChecker = {

  goCheck: function (e) {
    if ( mediawrapUtils.getPref('mediawrapEnabled', true) ) {
      var player = new MediaWrapPlayer(e.target, e.view.document);
      mediawrapChecker.checkMedia(player);
      if (player.setPlaceholder)
        e.preventDefault();
    }
    e.stopPropagation();
  },

  checkMedia: function(player) {

    var aObj = player.target;

    switch (aObj.nodeName) {
      case "OBJECT":
        mediawrapConverter.checkObject(player);
        break;

      case "EMBED":
        if ( mediawrapUtils.getPref('convertEmbed', true) && player.document.contentType == "text/html")
          mediawrapConverter.checkEmbed(player);
        break;

      case "BGSOUND":
        if ( mediawrapUtils.getPref('convertBgSound', true) 
            && mediawrapUtils.getPref('autoStartBgSound', true) )
          mediawrapConverter.convertBgsound(player);
        break;

      case "IMG":
        if ( mediawrapUtils.getPref('convertDynsrcImg', true) )
          mediawrapConverter.convertDynsrcImg(player);
        break;
    }
  }
}

function MediaWrapPlayer(target, doc) {
  this.target = target;
  this.document = doc;
  this.charset = doc.characterSet;
  this.tPlayerName = null;
  this.sPlayerName = null;
  this.url = null;
  this.sUrl = null;
  this.tMimeType = null;
  this.sMimeType = null;
  this.vlcPlayerName = null;
  this.isAudio = false;
  this.setPlaceholder = false;
  this.defaultVideoWidth = "320";
  this.defaultAudioWidth = "100%";
  this.videoHeight = 0;
}

var mediawrapConverter = {

  init: function() {

    this.hasPlugins = {
      "NewWMP": false,
      "WMP": false,
      "RP":  false,
      "QT":  false,
      "VLC": false,
      "SWF": false,
      "DCR": false
    };

    for (var i = 0; i < navigator.plugins.length; i++) {
      var playerName = this.getPlayerNameFromPlugin(navigator.plugins[i].name);
      if ( playerName != "other" )
        this.hasPlugins[playerName] = true;
    }
  },

  isURIcharcter: /[!\$&#'\(-;=\?-Z_a-z~]/,
  isDigit16:     /[0-9A-Fa-f]/,

  os: navigator.platform.indexOf("Win") == 0 ? "win" : "other",

  fileExtensionsGenerated: ["php","asp","aspx","html","htm","jhtml","cgi","fcgi","cfm","pl","ps","rstp","rst"],

  defaultMimeType: {
    'WMP':   'application/x-mplayer2',
    'VLC':   'application/x-vlc-plugin',
    'RP':    'audio/x-pn-realaudio-plugin',
    'QT':    'video/quicktime',
    'SWF':   'application/x-shockwave-flash',
    'DCR':   'application/x-director',
    'other': 'otherType'
  },

  pluginsPage: {
    'WMP':   'http://www.microsoft.com/windows/windowsmedia/',
    'VLC':   'http://www.videolan.org',
    'RP':    'http://www.real.com/',
    'QT':    'http://www.apple.com/quicktime/download/',
    'SWF':   'http://www.macromedia.com/go/getflashplayer',
    'DCR':   'http://www.macromedia.com/shockwave/download/'
  },

  mimeTypeTable: {  // mimetype: [A/V, player1, player2, ...], extension
    // Windows Media
    'application/x-mplayer2': ['AV','WMP','VLC'],  // *
    'application/asx': ['A','WMP','VLC','RP'],  // *
    'application/x-ms-wmp': ['AV','WMP','VLC','RP'],  // Force v11 Player
    'video/x-ms-asf-plugin': ['AV','WMP','VLC'],  // *
    'video/x-ms-asf': ['AV','WMP','VLC','RP'],  // asf,asx,*
    'video/x-ms-wm': ['AV','WMP','VLC','RP'],  // wm,*
    'audio/x-ms-wma': ['A','WMP','VLC','RP'],  // wma,*
    'audio/x-ms-wax': ['A','WMP','VLC','RP'],  // wax,*
    'video/x-ms-wmv': ['AV','WMP','VLC','RP'],  // wmv,*
    'video/x-ms-wvx': ['AV','WMP','VLC','RP'],  // wvx,*
    'application/x-drm': ['AV','WMP'],  // nip
    'application/x-drm-v2': ['AV','WMP'],  // nip
    'audio/x-oleobject': ['A','WMP','VLC','RP'],
    // VLC Media
    'application/x-vlc-plugin': ['AV','VLC','WMP'],
    'application/mpeg4-iod': ['AV','VLC'],
    'application/mpeg4-muxcodetable': ['AV','VLC'],
    'application/x-ogg': ['AV','VLC'],
    'audio/mpeg4': ['A','VLC','WMP'],
    'video/mpeg4': ['AV','VLC','WMP'],
    'video/mpeg-system': ['AV','VLC'],
    'video/x-mpeg-system': ['AV','VLC'],
    // real
    'audio/x-pn-realaudio-plugin': ['AV','RP'],  // rpm,*
    'audio/x-pn-realaudio': ['AV','RP'],
    'video/vnd.vivo': ['AV','RP'],
    'audio/vnd.rn-realaudio': ['AV','RP'],
    'audio/vnd.rn-realmedia': ['AV','RP'],
    'application/vnd.rn-realmedia': ['AV','RP'],
    'icm.div6': ['AV','RP'],  // divx
    'application/vnd.rn-realplayer-javascript': ['AV','RP'],  // rpj
    // quick time
    'video/quicktime': ['AV','QT','VLC'],  // mov,qt,mqv,*
    'audio/mp4': ['A','QT','VLC','RP'],  // mp4
    'video/mp4': ['AV','QT','VLC','RP'],  // mp4
    'audio/x-m4a': ['A','QT','VLC','RP'],  // m4a
    'audio/x-m4p': ['A','QT','VLC','RP'],  // m4p
    'audio/x-m4b': ['A','QT','VLC','RP'],  // m4b
    'video/x-m4v': ['AV','QT','VLC','RP'],  // m4v
    'audio/mpeg': ['A','QT','VLC','WMP','RP'],  // mp3,swa
    'audio/x-mpeg': ['A','QT','VLC','WMP','RP'],  // mp3
    'audio/mp3': ['A','QT','WMP','VLC','RP'],  // mp3,swa
    'audio/x-mp3': ['A','QT','WMP','VLC','RP'],  // mp3,swa
    'audio/mpeg3': ['A','QT','WMP','VLC','RP'],  // mp3,swa
    'audio/x-mpeg3': ['A','QT','WMP','VLC','RP'],  // mp3,swa
    'application/x-mpeg': ['AV','QT','WMP','VLC','RP'],  // amc
    'video/3gpp': ['AV','QT','VLC','RP'],  // 3gp,3gpp
    'audio/3gpp': ['A','QT','VLC','RP'],  // 3gp,3gpp
    'video/3gpp2': ['AV','QT','VLC','RP'],  // 3g2,3gp2
    'audio/3gpp2': ['A','QT','VLC','RP'],  // 3g2,3gp2
    'video/sd-video': ['AV','QT'],  // sdv
    'audio/amr': ['A','QT','RP'],  // amr
    'audio/aac': ['A','QT','RP'],  // aac,adts
    'audio/x-aac': ['A','QT','RP'],  // aac,adts
    'audio/x-caf': ['A','QT'],  // caf
    'audio/x-dv': ['A','QT'],
    'audio/x-sd2': ['A','QT'],
    'video/x-mpeg': ['AV','QT','WMP','VLC','RP'],  // mpeg,mpg
    'video/mpeg': ['AV','QT','WMP','VLC','RP'],  // mpeg,mpg
    'video/mpg': ['AV','QT','WMP','VLC','RP'],  // mpeg,mpg
    'audio/vnd.qcelp': ['A','QT','RP'],  // qcp
    'audio/x-gsm': ['A','QT','RP'],  // gsm
    'application/sdp': ['AV','QT','RP'],  // DCR
    'application/x-sdp': ['AV','QT','RP'],  // DCR
    'application/x-rtsp': ['AV','QT','RP'],
    'application/smil': ['AV','QT','RP'],  // smi,smil
    'video/x-msvideo': ['AV','QT','WMP','VLC','RP'],  // avi,vfw
    'video/msvideo': ['AV','QT','WMP','VLC','RP'],  // avi,vfw
    'video/avi': ['AV','QT','WMP','VLC','RP'],  // avi,vfw
    'video/x-sgi-movie': ['AV','QT','WMP','VLC'],
    'video/flc': ['AV','QT'],  // flc,fli,cel
    // WaveMidiAuAif
    'audio/aiff': ['A','QT','WMP','VLC','RP'],  // aiff,aif,aifc,cdda
    'audio/x-aiff': ['A','QT','WMP','VLC','RP'],  // aiff,aif,aifc,cdda
    'audio/basic': ['A','QT','WMP','VLC','RP'],  // au,snd,ulw
    'audio/mid': ['A','QT','WMP','VLC','RP'],  // mid,midi,smf,kar
    'audio/midi': ['A','QT','WMP','VLC','RP'],  // mid,midi,smf,kar
    'audio/x-midi': ['A','QT','WMP','VLC','RP'],  // mid,midi,smf,kar
    'audio/x-wav': ['A','QT','WMP','VLC','RP'],  // wav,bwf
    'audio/wav': ['A','QT','WMP','VLC','RP'],  // wav,bwf
    'audio/wave': ['A','QT','WMP','VLC','RP'],  // wav,bwf
    'audio/x-pn-wav': ['A','QT','WMP','VLC','RP'],  // wav,bwf
    // pls,m3u
    'audio/scpls': ['A','QT','WMP','VLC','RP'],
    'audio/pls': ['A','QT','WMP','VLC','RP'],
    'audio/x-scpls': ['A','QT','WMP','VLC','RP'],
    'audio/m3u': ['A','QT','WMP','VLC','RP'],
    'audio/x-mpegurl': ['A','QT','WMP','VLC','RP'],
    'audio/mpegurl': ['A','QT','WMP','VLC','RP'],
    'interface/x-tcmp-pls': ['A','QT','WMP','RP'],
    // Google Video Viewer
    'application/x-google-vlc-plugin': ['AV','VLC','WMP'],
    // Shockwave Flash
    'application/x-shockwave-flash': ['AV','SWF'],  // swf
    'application/futuresplash': ['AV','SWF'],  // spl
    // Shockwave for Director
    'application/x-director': ['AV','DCR']  // dir,dxr,dcr
  },

  extsMimeTable: {
    'dll': 'application/x-mplayer2',
    'asr': 'application/x-mplayer2',
    'wpl': 'application/x-mplayer2',
    'wmd': 'application/x-mplayer2',
    'wmz': 'application/x-mplayer2',
    'mac': 'video/quicktime'
  },

  tagParamTable: [
    {'WMP':'autostart','QT':'autoplay','RP':'autostart','VLC':'autoplay','BG':'autostart','IMG':'autostart'},
    {'WMP':'autostart','QT':'autostart','RP':'autostart','VLC':'autostart'},
    {'WMP':'autoplay','QT':'autoplay','RP':'autostart','VLC':'autostart'},
    {'WMP':'playcount:10000:#','QT':'loop:true','RP':'loop:true','VLC':'loop:true','BG':'loop:-1:#','IMG':'loop:-1:#'},
    {'WMP':'playcount:10000:#','QT':'loop:true','RP':'loop:true','VLC':'loop:true','BG':'loop:infinite:#','IMG':'loop:infinite:#'},
    {'WMP':'playcount:1:#','QT':'loop:false','RP':'numloop:1:#','VLC':'loop:false'},
    {'WMP':'playcount','QT':'loop','RP':'numloop','VLC':'loop','BG':'loop','IMG':'loop'},
    //{'WMP':'showcontrols:false:#','QT':'controller:false:#','RP':'controls:none'},
    {'WMP':'showcontrols:true','QT':'controller:true','RP':'controls:controlpanel'},
    {'WMP':'displaybackcolor','QT':'bgcolor','RP':'backgroundcolor'},
    {'WMP':'uimode:invisible:#','QT':'hidden:true','RP':'hidden:true'},
    //{'WMP':'uimode:none:#','QT':'controller:false','RP':'controls:none'},
    {'WMP':'uimode:mini:#','QT':'controller:true','RP':'controls:controlpanel'},
    {'WMP':'uimode:full','QT':'controller:true','RP':'controls:tacctrl,statusbar,controlpanel'},
    {'WMP':'enabled:true','QT':'controller:true','RP':'controls:controlpanel'},
    {'WMP':'showaudiocontrols:true','QT':'controller:true','RP':'controls:controlpanel'},
    {'WMP':'enablepositioncontrols:true','QT':'controller:true','RP':'controls:positionfield'},
    {'WMP':'showpositioncontrols:true','QT':'controller:true','RP':'controls:positionfield'},
    {'WMP':'showstatusbar:true','QT':'controller:true','RP':'controls:statusbar'},
    {'WMP':'stretchtofit:true','QT':'scale:aspect:#','RP':'maintainaspect:true'},
    {'WMP':'stretchtofit:true','QT':'scale:tofit','RP':'maintainaspect:true'},
    {'WMP':'volume','QT':'volume','BG':'volume','VLC':'volume'},
    {'WMP':'mute:true','QT':'volume:0'}
  ],

  globalAttributes: ['id','name','width','height','hidden','pluginspage',
                     'align','border','hspace','vspace','class','dir','style','title'],

  dropAttributes: ['src','data','url','filename','movie','codebase','classid','standby',
                   'samistyle','samilang','type'],

  boolTagParam: {
    'WMP': /^(?:auto|allow|enable|show|fullscreen|mute|clicktoplay|stretchtofit|invokeurls)/,
    'RP':  /^(?:autogotourl|autostart|center|loop|maintainaspect|prefetch|shuffle)$/,
    'QT':  /^(?:autohref|autoplay|autostart|controller|loop|kioskmode)$/,
    'VLC': /^(?:auto|enable|show|visible|loop|fullscreen|mute)/,
    'SWF': /^(?:swliveconnect|play|loop|menu)$/,
    'BG':  /^(?:autostart)$/,
    'IMG': /^(?:autostart)$/
  },

  isBoolTagParam: function(paramName, playerName) {
    if (this.boolTagParam[playerName])
      return this.boolTagParam[playerName].test(paramName);
    else
      return false;
  },

  isFileExtensionsGenerated: function(ext) {
    return this.isInConcourse(ext, this.fileExtensionsGenerated)
  },

  isGlobalAttribute: function(attrName) {
    return this.isInConcourse(attrName, this.globalAttributes)
  },

  isDropAttribute: function(attrName) {
    return this.isInConcourse(attrName, this.dropAttributes)
  },

  isInConcourse: function(o, concourse) {
    for (var i = 0; i < concourse.length; i++)
      if (o == concourse[i]) return true;
    return false;
  },

  getPlayerNameFromPlugin: function(pluginName) {
    if (pluginName.indexOf("Windows Media Player Firefox Plugin")>-1) {
      this.hasPlugins["NewWMP"] = true;
      return "WMP";
    }
    else if (pluginName.indexOf("Windows Media")>-1 || pluginName.indexOf("Flip4Mac")>-1)
      return "WMP";
    else if (pluginName.indexOf("RealPlayer")>-1)
      return "RP";
    else if (pluginName.indexOf("QuickTime")>-1)
      return "QT";
    else if (pluginName.indexOf("VLC")>-1)
      return "VLC";
    else if (pluginName.indexOf("Shockwave Flash")>-1)
      return "SWF";
    else if (pluginName.indexOf("Director")>-1)
      return "DCR";
    else
      return "other";
  },

  getDefaultMimeType: function(playerName) {
    return this.defaultMimeType[playerName];
  },

  getPlayerNameFromMimeType: function(type) {
    if (/application\/x\-java\-/.test(type)) return "";
    try {
      return this.getPlayerNameFromPlugin(navigator.mimeTypes[type].enabledPlugin.name);
    } catch(e) {
      return "";
    }
  },

  selectPlayerFromMimeType: function(player, mimeType) {
    player.tPlayerName = this.getPlayerNameFromMimeType(mimeType);
    if (player.tPlayerName) {
      player.tMimeType = mimeType;
      return true;
    }
    else if (this.mimeTypeTable[mimeType]) {
      // the sourceMimeType is in mimeTypeTable
      var playerName, dType;
      for (var i = 1; i < this.mimeTypeTable[mimeType].length; i++) {
        // find the player in order
        playerName = this.mimeTypeTable[mimeType][i];
        dType = this.getDefaultMimeType(playerName);
        player.tPlayerName = this.getPlayerNameFromMimeType(dType);
        if (player.tPlayerName && this.hasPlugins[player.tPlayerName]) {
          // find the player and firefox have plugin
          player.tMimeType = this.getDefaultMimeType(player.tPlayerName);
          return true;
        }
      }
    }
    player.tPlayerName = "other";
    player.tMimeType = mimeType;
    return false;
  },

  getFileNameFromURL: function (url) {
    if (!url) return "";
    if (url.indexOf("?") > -1)
      url = url.substring(0, url.indexOf("?"));
    return url.substr(url.lastIndexOf("/") + 1);
  },

  getExtFromURL: function(url) {
    var fileName = this.getFileNameFromURL(url);
    if (!fileName || fileName.indexOf(".") == -1) return "";
    var ext = fileName.substr(fileName.lastIndexOf(".") + 1);
    return ext ? ext.toLowerCase() : "";
  },

  getMimeTypeFromExt: function(ext) {
    if (!ext) return "";
    var mimeSvc = Components.classes["@mozilla.org/mime;1"].
                    getService(Components.interfaces.nsIMIMEService);
    try {
      return mimeSvc.getTypeFromExtension(ext).toLowerCase();
    }
    catch (e) {
      return this.extsMimeTable[ext];
    }
  },

  smartSelectPlayer: function(player, isAx) {
    var mimeType = player.sMimeType;
    var obj = player.target;
    if ( !(player.url || obj.id || ("EMBED"==obj.nodeName ? obj.name : false)) ||
        !mediawrapUtils.getPref('autoSelectPlayer', true) )
      return;

    var ext = this.getExtFromURL(player.url);

    // in windows and is ActiveX object, convert the original mimetype to default mimetype
    if (isAx && this.os == "win" && player.sPlayerName != "BG" && player.sPlayerName != "IMG") {
      if (this.selectPlayerFromMimeType(player, mimeType)) {
        player.tMimeType = this.getDefaultMimeType(player.tPlayerName);
        if (player.tMimeType || this.isFileExtensionsGenerated(ext))
          return;
        // no player from original mimetype, then find player from extension
        var type = this.getMimeTypeFromExt(ext);
        if (type && !this.selectPlayerFromMimeType(player, type)) {
          // no player
          player.tPlayerName = "other";
          player.tMimeType = mimeType;
        }
        return;
      }
    }

    var type = this.getMimeTypeFromExt(ext);
    if (!type || this.isFileExtensionsGenerated(ext)) {
      if (this.selectPlayerFromMimeType(player, mimeType)) return;
    }
    if (type && !this.selectPlayerFromMimeType(player, type)) {
      player.tPlayerName = "other";
      player.tMimeType = mimeType;
    }
    return;
  },

  checkAudio: function(player, aType) {

    var ext = this.getExtFromURL(player.url);
    if (ext) {
      var mType = this.getMimeTypeFromExt(ext);
      if (this.mimeTypeTable[mType]) {
        player.isAudio = ("A" == this.mimeTypeTable[mType][0]);
        return;
      }
    }

    if (this.mimeTypeTable[aType])
      player.isAudio = ("A" == this.mimeTypeTable[aType][0]);
    else
      player.isAudio = false;

    return;
  },

  // get the source player name & mimetype
  checkPlayer: function(player) {
    player.sMimeType = player.target.getAttribute("type");

    this.getURL(player);

    player.sPlayerName = this.getPlayerNameFromTypeTable(player.sMimeType);
    if (player.sPlayerName)
      return;

    var ext = this.getExtFromURL(player.url);
    if (ext && !this.isFileExtensionsGenerated(ext)) {
      player.sMimeType = this.getMimeTypeFromExt(ext);
      player.sPlayerName = this.getPlayerNameFromTypeTable(player.sMimeType);
      if (player.sPlayerName)
        return;
    }

    player.sPlayerName = "other";
    player.sMimeType = "otherType";
  },

  getPlayerNameFromTypeTable: function(type) {
    try {
      return this.mimeTypeTable[type.toLowerCase()][1];
    } catch(e) {
      return "";
    }
  },

  gURL: function (player) {
    var obj = player.target;

    switch (obj.nodeName) {
      case "OBJECT":
        if (!obj.getAttribute("classid"))
          return obj.getAttribute("data") || obj.getAttribute("src");
        var pName, value;
        var params = obj.getElementsByTagName("param");
        for (var p = 0; p < params.length; p++) {
          pName = params[p].getAttribute("name");
          value = params[p].getAttribute("value");
          if (!pName || !value) continue;
          pName = pName.toLowerCase();
          if (pName=="src" || pName=="filename" || pName=="url" || pName=="movie")
            return value;
        }
        return "";
      case "EMBED":
        return obj.getAttribute("src") || obj.getAttribute("filename") || obj.getAttribute("target");
      case "BGSOUND":
        return obj.getAttribute("src");
      case "IMG":
        var url = obj.getAttribute("dynsrc");
        obj.removeAttribute("dynsrc");
        return url;
    }
    return "";
  },

  getURL: function (player) {
    var url = this.gURL(player);
    if (!url) return;
    if (player.charset)
      player.url = player.sUrl = this.convertURL(player, url);
    else
      player.url = player.sUrl = url;
  },

  convertURL: function (player, aUrl) {
    if (!aUrl || this.isLocalLink(player, aUrl)) return aUrl;
    return this.encodeURI(player, aUrl);
  },

  isLocalLink: function (player, aUrl) {
    if (!aUrl) return false;
    var index = aUrl.indexOf("://");
    if (index > -1 && index < 8) {
      if (aUrl.indexOf("file:///")==0)
        return true;
    }
    else {
      var href = player.document.location.href;
      if ( href && href.indexOf("file:///") == 0 )
        return true;
    }
    return false;
  },

  creatRpmURL: function (player) {
    var url = player.url;
    if ( !url || /^data:/i.test(url) ) return;
    var ext = this.getExtFromURL(url);
    if ( "rpm" == ext || "ram" == ext || "rpj" == ext || "rm" == ext || "rmvb" == ext ||
          this.isFileExtensionsGenerated(ext) )
      return;
    this.getFullURL(player);
    player.url = "data:text/plain," + player.url;
  },

  getFullURL: function (player) {
    var url = player.url;
    if ( !url || url.indexOf("://") > 0 && url.indexOf("://") < 8 )
      return;
    var href = player.document.location.href;
    if (/^\/+/.test(url))
      href = href.substring(0, href.search(/[^:\/]\//)+1);
    else 
      href = href.substring(0, href.lastIndexOf("/")+1);
    player.url = href + url;
  },

  encodeURI: function (player, aUrl) {
    var converter = Components.classes["@mozilla.org/intl/saveascharset;1"].
            createInstance(Components.interfaces.nsISaveAsCharset);
    converter.Init(player.charset, null, null);

    aUrl = aUrl.replace(/^\s+/,"").replace(/\s+$/,"").replace(/\\/g,"/");

    var str, url = "";
    var ss = aUrl.replace(/(\/+)/g,"$1/|/").split(/\/\|\//);
    for ( var i = 0; i < ss.length; i++ ) {
      try {
        str = decodeURI(ss[i]);
      } 
      catch(e) {
        str = ss[i];
      }

      if ( this.needEncode(str) ) {
        try {
          str = converter.Convert(str);
        } catch(e) {}
        var s;
        for (var j = 0; j < str.length; j++) {
          s = str.charAt(j);
          if ( this.isURIcharcter.test(s) ) {
            url += s;
          }
          else {
            if ("%" == s) {
              // do not encode "%" if there are 2 digital following it.
              // thanks for Chang Yung-Ting <foreverr@gmail.com> to patch it.
              var s2 = str.charAt(j+1);
              if (this.isDigit16.test(s2)) {
                s2 = str.charAt(j+2);
                if (this.isDigit16.test(s2)) {
                  url += s;
                  continue;
                }
              }
            }
            url += "%" + str.charCodeAt(j).toString(16);
          }
        }
      }
      else {
        url += str;
      }
    }
    return url;
  },

  needEncode: function (str) {
    var s;
    for (var i = 0; i < str.length; ) {
      s = str.charAt(i++);
      if (this.isURIcharcter.test(s)) continue;
      if ("%" == s) {
        s = str.charAt(i++);
        if (this.isDigit16.test(s)) {
          s = str.charAt(i++);
          if (this.isDigit16.test(s)) continue;
        }
      }
      return true;
    }
    return false;
  },

  checkObject: function(player) {
    player.sPlayerName = "other";
    var obj = player.target;
    if (obj.getAttribute("classid")) {
      switch (obj.getAttribute("classid").toLowerCase()) {
        case "clsid:05589fa1-c356-11ce-bf01-00aa0055595a":  // wmp6
        case "clsid:22d6f312-b0f6-11d0-94ab-0080c74c7e95":  // wmp6.4
        case "clsid:6bf52a52-394a-11d3-b153-00c04f79faa6":  // wmp7&9&10
          if ( mediawrapUtils.getPref('convertWindowsMediaAx', true) ) {
            player.sPlayerName = "WMP";
          }
          break;
        case "clsid:cfcdaa03-8be4-11cf-b84b-0020afbbccfa":  // real player
          if ( mediawrapUtils.getPref('convertRealAx', true) ) {
            player.sPlayerName = "RP";
          }
          break;
        case "clsid:02bf25d5-8c17-4b23-bc80-d3488abddc6b":  // quicktime player
          if ( mediawrapUtils.getPref('convertQuickTimeAx', true) ) {
            player.sPlayerName = "QT";
          }
          break;
        case "clsid:d27cdb6e-ae6d-11cf-96b8-444553540000":  // shockwave flash player
          if ( mediawrapUtils.getPref('convertShockwaveFlashAx', true) ) {
            player.sPlayerName = "SWF";
          }
          break;
        case "clsid:166b1bca-3f9c-11cf-8075-444553540000":  // shockwave Director player
          if ( mediawrapUtils.getPref('convertShockwaveDirectorAx', true) ) {
            player.sPlayerName = "DCR";
          }
          break;
      }
      player.sMimeType = this.getDefaultMimeType(player.sPlayerName);

      if (player.sPlayerName != "other") {
        this.convertObjectAx(player);
      }
      /*else if ( mediawrapUtils.getPref('convertEmbed', true) ) {
        this.triggerEmbed(obj);
      }*/
    }
    else if ( mediawrapUtils.getPref('convertObjectNoClassId', true) ) {
      this.convertObjNoClassId(player);
    }
  },

  triggerEmbed: function (obj) {
    var embs = obj.getElementsByTagName("embed");
    for ( var i = embs.length-1; i >= 0; i-- ) {
      try {embs[i].getAttribute("src")} catch(e){}
    }
  },

  checkEmbed: function(player) {
    this.convertObjNoClassId(player);
  },

  convertObjNoClassId: function(player) {
    this.checkPlayer(player);
    if ( player.sPlayerName != "other" )
      this.convertObject(player);
    else
      this.triggerEmbed(player.target);
  },

  convertObjectAx: function (player) {
    this.getURL(player);
    this.convertObject(player, true);
  },

  convertObject: function(player, isAx) {
    var obj = player.target;
    if (!player.url) {
      // no URL,check the inside embed tag.
      // thanks for Chang Yung-Ting <foreverr@gmail.com>
      var emb = obj.getElementsByTagName("embed")[0];
      if (emb && (emb.getAttribute("src") || emb.getAttribute("filename") || emb.getAttribute("target")))
        return;
    }

    this.checkAudio(player, player.sMimeType);
    this.smartSelectPlayer(player, isAx);
    this.checkSize(player);
    this.covertPlayer(player);
  },

  convertBgsound: function (player) {
    player.sPlayerName = "BG";
    player.sMimeType = "other";
    player.isAudio = true;

    this.getURL(player);
    if (!player.url) return;

    this.smartSelectPlayer(player);

    if (player.tPlayerName && player.tPlayerName != "BG") {
      var bgsd = player.target;
      bgsd.setAttribute("autostart","1");
      bgsd.setAttribute("width","0");
      bgsd.setAttribute("height","0");
      var volume = bgsd.getAttribute("volume");
      bgsd.setAttribute("volume", this.convertVolume(volume));
      this.covertPlayer(player);
    }
  },

  convertDynsrcImg: function (player) {
    player.sPlayerName = "IMG";
    player.sMimeType = "other";
    player.isAudio = false;

    this.getURL(player);
    if (!player.url) return;

    this.smartSelectPlayer(player);

    if (player.tPlayerName && player.tPlayerName != "IMG") {
      this.checkSize(player);
      this.covertPlayer(player);
    }
  },

  covertPlayer: function (player) {

    var attrName, attrValue;
    var obj = player.target;
    var tPlayerName = player.tPlayerName;

    if (this.pluginsPage[tPlayerName])
      obj.setAttribute("pluginspage", this.pluginsPage[tPlayerName]);

    if (tPlayerName == "RP")
      this.creatRpmURL(player);

    if (tPlayerName == "WMP" && player.url && player.url.indexOf("?") >= 0)
      this.getFullURL(player);

    /*if (obj.nodeName == "EMBED" && (tPlayerName != "WMP") && (tPlayerName != "RP") && (tPlayerName != "QT") &&
            (player.sMimeType == player.tMimeType) && (player.url == player.sUrl)) {
      this.setNewParams(player);
    }
    else*/ {
      obj.removeAttribute("data");
      obj.removeAttribute("src");
      //obj.removeAttribute("type");

      this.convertObjectToEmbed(player);
      var newObj = player.target;

      if (tPlayerName == player.sPlayerName) {
        this.setNewParams(player);
      }
      else {
        for (var i = 0; i < newObj.attributes.length; i++) {
          attrName = newObj.attributes[i].name.toLowerCase();
          attrValue = newObj.attributes[i].value;
          if ( !this.isGlobalAttribute(attrName) && attrValue != null)
            this.convertAttrToAnother(player, attrName, attrValue);
        }

        if (tPlayerName == "RP") {
          if (!newObj.getAttribute("controls"))
            newObj.setAttribute("controls", "default");
          this.checkSize(player);
        }

        this.setNewParams(player);

        if (tPlayerName == "RP" && !player.isAudio) {
          newObj.setAttribute("height", this.getControlHeight(player, "RP"));
          newObj.setAttribute("console", "clip_" + (new Date()).getTime());
          var newObj1 = newObj.cloneNode(true);
          newObj1.setAttribute("controls","ImageWindow");
          newObj1.setAttribute("height", player.videoHeight);
          obj.parentNode.insertBefore(newObj1, obj);
          obj.parentNode.insertBefore(player.document.createElement("br"), obj);
        }
      }

      // remove the orginal player
      obj.setAttribute("mediawrapChecked", "removed");

      // for set placeholder
      obj.id = newObj.id;
      obj.setAttribute("mediawrapType", newObj.getAttribute("mediawrapType"));

      obj.parentNode.insertBefore(newObj, obj);
    }
  },

  convertObjectToEmbed: function(player) {
    var obj = player.target;
    var newObj = player.document.createElement("embed");
    player.target = newObj;

    var name, value;
    for (var i = 0; i < obj.attributes.length; i++) {
      name = obj.attributes[i].name.toLowerCase();
      value = obj.attributes[i].value;
      if (name == "standby" && player.sPlayerName == "WMP")
        newObj.setAttribute("showstatusbar", "1");
      else if ( !this.isDropAttribute(name) && !(player.sPlayerName == "VLC" && name == "target") )
        try {newObj.setAttribute(name, value);} catch(e) {}
    }

    if ( obj.nodeName != "OBJECT" ) return;

    var params = obj.getElementsByTagName("param");
    for (var p = 0; p < params.length; p++) {
      name = params[p].getAttribute("name");
      value = params[p].getAttribute("value");

      // Unavailability params can crash the firefox, does not copy it.
      if ( !name || !value || /[^\w\-]/i.test(name) ) continue;

      name = name.toLowerCase();
      if ( this.isDropAttribute(name) ) continue;

      if ( "base" == name || "baseurl" == name || "href" == name || "qtsrc" == name
            || "urlsubstitute" == name || "samifilename" == name
            || name.indexOf("hotspot") == 0 || name.indexOf("qtnext") == 0 ) {
        newObj.setAttribute(name, this.convertURL(player, value));
      }
      else if ("WMP" == player.sPlayerName && name == "playcount") {
        value = parseInt(value);
        value = isNaN(value) ? "loop" : (value > 1 ? value : 1);
        newObj.setAttribute("playcount", value);
      }
      else if (name == "volume") {
        newObj.setAttribute("volume", this.convertVolume(value));
      }
      else if ( this.isBoolTagParam(name, player.sPlayerName) ) {
        value = this.convertFromBool(value, player.sPlayerName);
        newObj.setAttribute(name, this.convertToBool(value, player.sPlayerName));
      }
      else
        newObj.setAttribute(name, value);
    }

    // copy the inside embed tag's "name" attribute to newObj
    obj.setAttribute("mediawrapChecked", "checking");
    var emb = obj.getElementsByTagName("embed")[0];
    if (emb && emb.hasAttribute("name"))
      newObj.setAttribute("name", emb.getAttribute("name"));
    else if (!newObj.hasAttribute("name") && newObj.id)
      newObj.setAttribute("name", newObj.id);
  },

  convertVolume: function (volume) {
    if (!volume) return 100;
    volume = parseInt(volume);
    if (volume < 0)
      volume = parseInt((volume + 10000)/100);
    return (volume > 100) ? 100 : volume;
  },

  convertFromBool: function (value, playerName) {
    if (playerName == "VLC")
      return /^(?:yes|true)$/i.test(value) ? "true" : (new Boolean(parseInt(value))).toString();
    else
      return /^true$/i.test(value) ? "true" : (new Boolean(parseInt(value))).toString();
  },

  convertToBool: function (value, playerName) {
    if (playerName == "WMP")
      return /^false$/i.test(value) ? "0" : ((parseFloat(value) == 0) ? "0" : "1");
    else
      return /^true$/i.test(value) ? "true" : (new Boolean(parseInt(value))).toString();
  },

  convertAttrToAnother: function(player, attrName, attrValue) {
    var attr, newAttr;
    var obj = player.target;
    var sName = player.sPlayerName;
    var tName = player.tPlayerName;

    for (var i = 0; i < this.tagParamTable.length; i++) {
      if (!this.tagParamTable[i][sName]) continue;
      attr = this.tagParamTable[i][sName].split(":");
      if (attr[0] != attrName) continue;
      if (this.isBoolTagParam(attrName, sName)) {
        attrValue = this.convertFromBool(attrValue, sName);
      }
      else
        attrValue = attrValue.toLowerCase();
      if (attr[1] == null || attr[1] == attrValue) {
        if (!this.tagParamTable[i][tName]) break;
        newAttr = this.tagParamTable[i][tName].split(":");
        if (newAttr[1])
          attrValue = newAttr[1];
        obj.removeAttribute(attrName);
        if (this.isBoolTagParam(newAttr[0], tName))
          obj.setAttribute(newAttr[0], this.convertToBool(attrValue, tName));
        else {
          if (obj.getAttribute(newAttr[0]))
            attrValue = this.concatAttr(obj.getAttribute(newAttr[0]), attrValue);
          obj.setAttribute(newAttr[0], attrValue);
        }
        return;
      }
      else if (attr[2] != "#")
        break;
    }
  },

  concatAttr: function(attr1, attr2) {
    var s = attr1;
    var s1 = attr1.split(",");
    var s2 = attr2.split(",");
    for (var i = 0; i < s1.length; i++) {
      s1[i] = s1[i].replace(/^\s+/,"").replace(/\s+$/,"");
      for (var j = 0; j < s2.length; j++) {
        s2[j] = s2[j].replace(/^\s+/,"").replace(/\s+$/,"");
        if (s1[i] == s2[j])
          s2[j] = "";
      }
    }
    for (var j = 0; j < s2.length; j++) {
      if (s2[j]) {
        s += (s ? "," : "") + s2[j];
      }
    }
    return s;
  },

  setNewParams: function(player) {
    var obj = player.target;

    obj.setAttribute("mediawrapChecked","true");

    if (player.url) {
      obj.setAttribute("src", player.url);
      if (player.tPlayerName == "VLC")
        obj.setAttribute("target", player.url);
    }
    if (player.tMimeType)
      obj.setAttribute("type", player.tMimeType);
    if (!obj.id)
      obj.id = "Player" + (new Date()).getTime();
    var str = obj.id + '= document.getElementById("' + obj.id + '");';
    this.execScript(player, str);

    if (player.sPlayerName) {
      obj.setAttribute("sPlayerName", player.sPlayerName);
      if (player.sPlayerName == "WMP" && (/wmp/i.test(player.tPlayerName))) {
        obj.setAttribute("mediawrapChecked","initWmp");
        obj.style.display = "none";
      }
    }
    if (player.tPlayerName) {
      if (player.tPlayerName == "WMP" && this.hasPlugins["NewWMP"])
        obj.setAttribute("tPlayerName", "NewWMP");
      else
        obj.setAttribute("tPlayerName", player.tPlayerName);
    }

    this.showWmpControl(player);
    this.checkAutoStart(player);
  },

  showWmpControl: function (player) {

    if (player.tPlayerName != "WMP") return;

    var obj = player.target;

    if ( mediawrapUtils.getPref('alwaysShowWmpControl', true) ) {
      var attrNames = ["enabled","showcontrols","showaudiocontrols"];
      for (var i = 0; i < attrNames.length; i++) {
        obj.removeAttribute(attrNames[i]);
      }
      var attrValue = obj.getAttribute("uimode");
      if (attrValue && "none" == attrValue.toLowerCase())
        obj.setAttribute("uimode", "full");
    }

    if ( mediawrapUtils.getPref('alwaysShowWmpContextMenu', true) ) {
      obj.removeAttribute("enablecontextmenu");
    }
  },

  checkAutoStart: function (player) {
    var obj = player.target;

    if ("WMP" == player.tPlayerName && !obj.getAttribute("autostart") && obj.getAttribute("autoplay")) {
      var astart = this.convertFromBool(obj.getAttribute("autoplay"), "WMP");
      obj.removeAttribute("autoplay");
      obj.setAttribute("autostart", this.convertToBool(astart, "WMP"));
    }

    if ( this.isLocalLink(player, player.url) && mediawrapUtils.getPref('autoStartLocalLinks', true) )
      return;

    if ( "SWF"==player.tPlayerName ) {
      if (!mediawrapUtils.getPref('autoStartFlash', true)) {
        obj.setAttribute("mediawrapType", "flash");
        this.replaceURL(player);
        player.setPlaceholder = true;
      }
      return;
    }

    if ( "DCR"==player.tPlayerName ) {
      if (!mediawrapUtils.getPref('autoStartDirector', true)) {
        obj.setAttribute("mediawrapType", "director");
        this.replaceURL(player);
        player.setPlaceholder = true;
      }
      return;
    }

    if ( mediawrapUtils.getPref('autoStartEnabled', true) ) return;

    if ( mediawrapUtils.getPref('autoStartBgSound', true) ) {
      if ( obj.getAttribute("hidden") && obj.getAttribute("hidden").toLowerCase() == "true"
            || obj.getAttribute("height") == "0" || obj.getAttribute("width") == "0" )
        return;
    }

    switch (player.tPlayerName) {
      case "QT":
      case "VLC":
        obj.setAttribute("autoplay", "false");
        obj.removeAttribute("autostart");
        break;
      default:
        obj.setAttribute("autostart", this.convertToBool("false",player.tPlayerName));
        obj.removeAttribute("autoplay");
    }
  },

  replaceURL: function (player) {
    var obj = player.target;

    if (player.url) {
      obj.setAttribute("origSrc", player.url);
      obj.removeAttribute("src");
    }
  },

  getRealSize: function (player, sizeName) {
    var obj = player.target;
    var value = obj.getAttribute(sizeName);
    if (!value) {
      value = player.document.defaultView.getComputedStyle(obj, null).getPropertyValue(sizeName);
      value = /^(\d+%?)(?:px)?$/.test(value) ? RegExp.$1 : "";
      if (value) {
        obj.setAttribute(sizeName, value);
      }
    }
    return value;
  },

  checkSize: function (player) {
    var obj = player.target;

    var hide = obj.getAttribute("hidden");
    if ( hide && hide.toLowerCase() == "true" || obj.hasAttribute("hidden") )
      return;

    var width = this.getRealSize(player, "width");
    var height = this.getRealSize(player, "height");

    if (width && height && player.sPlayerName == player.tPlayerName) return;

    if (width && width.indexOf("%")==-1 && height && height.indexOf("%")==-1) {
      if (player.isAudio) {
        var hgt = this.getControlHeight(player, player.tPlayerName);
        obj.setAttribute("height", (height > hgt ? height : hgt));
      }
      else {
        width = parseInt(width);
        player.videoHeight = height = this.getVideoHeight(player, parseInt(height));
        if (height < 50 || width/height >= 4) {
          player.isAudio = true;
          height = 0;
        }
        height += this.getControlHeight(player, player.tPlayerName)
        obj.setAttribute("height", height);
      }
    }
    else if ((!width || width.indexOf("%")>-1)&&(!height || height.indexOf("%")>-1)) {
      if (player.isAudio) {
        if (!width) obj.setAttribute("width", "100%");
        obj.setAttribute("height", this.getControlHeight(player, player.tPlayerName));
      }
      else {
        if (/^\s*(\d{1,3})%/.test(width)) {
          width = parseInt(parseInt(obj.parentNode.clientWidth)*parseInt(RegExp.$1)/100);
          if (width > player.defaultVideoWidth)
            width = player.defaultVideoWidth;
        }
        else {
          width = obj.parentNode.clientWidth;
          if (width > player.defaultVideoWidth) {
            width = player.defaultVideoWidth;
            obj.setAttribute("width", player.defaultVideoWidth);
          }
          else
            obj.setAttribute("width", "100%");
        }
        player.videoHeight = parseInt(parseInt(width)*3/4);
        obj.setAttribute("height", 
            player.videoHeight + this.getControlHeight(player, player.tPlayerName));
      }
    }
    else if (width && width.indexOf("%")==-1) {
      if (player.isAudio) {
        obj.setAttribute("height", this.getControlHeight(player, player.tPlayerName));
      }
      else {
        player.videoHeight = parseInt(parseInt(width)*3/4);
        height = player.videoHeight + this.getControlHeight(player, player.tPlayerName);
        obj.setAttribute("height", height);
      }
    }
    else {
      player.videoHeight = height = this.getVideoHeight(player, parseInt(height));
      if (height < 50) {
        player.isAudio = true;
        height = 0;
      }
      height += this.getControlHeight(player, player.tPlayerName)
      obj.setAttribute("height", height);
      obj.setAttribute("width", "100%");
    }
  },

  getVideoHeight: function (player, height) {
    var ctrlHeight = this.getControlHeight(player, player.sPlayerName);
    ctrlHeight = parseInt(ctrlHeight*height/(ctrlHeight+height));
    return ((height - ctrlHeight)<1) ? 1 : (height - ctrlHeight);
  },

  getControlHeight: function (player, aPlayerName) {
    var obj = player.target;
    var ctrlHeight = 0;

    switch (aPlayerName) {
      case "WMP":
        if ( !(obj.getAttribute("enabled")||obj.getAttribute("showcontrols")
          || obj.getAttribute("showaudiocontrols")||obj.getAttribute("uimode"))
          || this.convertFromBool(obj.getAttribute("enabled")) == "true"
          || this.convertFromBool(obj.getAttribute("showcontrols")) == "true"
          || this.convertFromBool(obj.getAttribute("showaudiocontrols")) == "true"
          || (obj.getAttribute("uimode") && (obj.getAttribute("uimode").toLowerCase() == "mini"
          || obj.getAttribute("uimode").toLowerCase() == "full")) )

          ctrlHeight += 44;

        if (this.convertFromBool(obj.getAttribute("showstatusbar")) == "true")
          ctrlHeight += 23;

        return ctrlHeight;

      case "RP":
        var ctrlValue = obj.getAttribute("controls");

        if (!ctrlValue) return ctrlHeight;

        if (/(?:ControlPanel|all|default|FFCtrl|HomeCtrl|MuteCtrl|PauseButton|PlayButton|PlayOnlyButton|RWCtrl|StopButton)/i.test(ctrlValue))
          ctrlHeight += 24;  //36
        if (/(?:StatusBar|all|default|PositionSlider|PositionField|StatusField)/i.test(ctrlValue))
          ctrlHeight += 20;  //30
        if (/(?:TACCtrl|all|default)/i.test(ctrlValue))
          ctrlHeight += 20;  //32
        if (/(?:InfoPanel|InfoVolumePanel|VolumeSlider|MuteVolume)/i.test(ctrlValue))
          ctrlHeight += 40;  //55
        return ctrlHeight;

      case "QT":
        if (!obj.getAttribute("controller") || 
            this.convertFromBool(obj.getAttribute("controller")) == "true")
          ctrlHeight += 16;
        return ctrlHeight;

      default: 
        return ctrlHeight;
    }
  },

  execScript: function (player, text) {
    var scriptElm = player.document.createElement("script");
    scriptElm.appendChild(player.document.createTextNode(text));
    if (player.document.body) {
      player.document.body.appendChild(scriptElm);
      player.document.body.removeChild(scriptElm);
    }
  }
}

mediawrapConverter.init();

window.addEventListener("mediawrapCheckLoad", mediawrapChecker.goCheck, true, true);